@@ -1,17 +1,21 @@
 """Quickly setup documentation source to work with Sphinx."""
 
+from __future__ import annotations
+
 import argparse
 import locale
 import os
 import sys
 import time
-from collections import OrderedDict
 from os import path
-from typing import Any, Callable, Dict, List, Union
+from typing import TYPE_CHECKING, Any, Callable
 
 # try to import readline, unix specific enhancement
 try:
     import readline
+    if TYPE_CHECKING and sys.platform == "win32":  # always false, for type checking
+        raise ImportError
+    READLINE_AVAILABLE = True
     if readline.__doc__ and 'libedit' in readline.__doc__:
         readline.parse_and_bind("bind ^I rl_complete")
         USE_LIBEDIT = True
@@ -19,7 +23,7 @@ try:
         readline.parse_and_bind("tab: complete")
         USE_LIBEDIT = False
 except ImportError:
-    readline = None
+    READLINE_AVAILABLE = False
     USE_LIBEDIT = False
 
 from docutils.utils import column_width
@@ -31,18 +35,18 @@ from sphinx.util.console import bold, color_terminal, colorize, nocolor, red  #
 from sphinx.util.osutil import ensuredir
 from sphinx.util.template import SphinxRenderer
 
-EXTENSIONS = OrderedDict([
-    ('autodoc', __('automatically insert docstrings from modules')),
-    ('doctest', __('automatically test code snippets in doctest blocks')),
-    ('intersphinx', __('link between Sphinx documentation of different projects')),
-    ('todo', __('write "todo" entries that can be shown or hidden on build')),
-    ('coverage', __('checks for documentation coverage')),
-    ('imgmath', __('include math, rendered as PNG or SVG images')),
-    ('mathjax', __('include math, rendered in the browser by MathJax')),
-    ('ifconfig', __('conditional inclusion of content based on config values')),
-    ('viewcode', __('include links to the source code of documented Python objects')),
-    ('githubpages', __('create .nojekyll file to publish the document on GitHub pages')),
-])
+EXTENSIONS = {
+    'autodoc': __('automatically insert docstrings from modules'),
+    'doctest': __('automatically test code snippets in doctest blocks'),
+    'intersphinx': __('link between Sphinx documentation of different projects'),
+    'todo': __('write "todo" entries that can be shown or hidden on build'),
+    'coverage': __('checks for documentation coverage'),
+    'imgmath': __('include math, rendered as PNG or SVG images'),
+    'mathjax': __('include math, rendered in the browser by MathJax'),
+    'ifconfig': __('conditional inclusion of content based on config values'),
+    'viewcode': __('include links to the source code of documented Python objects'),
+    'githubpages': __('create .nojekyll file to publish the document on GitHub pages'),
+}
 
 DEFAULTS = {
     'path': '.',
@@ -127,10 +131,12 @@ def ok(x: str) -> str:
     return x
 
 
-def do_prompt(text: str, default: str = None, validator: Callable[[str], Any] = nonempty) -> Union[str, bool]:  # NOQA
+def do_prompt(
+    text: str, default: str | None = None, validator: Callable[[str], Any] = nonempty,
+) -> str | bool:
     while True:
         if default is not None:
-            prompt = PROMPT_PREFIX + '%s [%s]: ' % (text, default)
+            prompt = PROMPT_PREFIX + f'{text} [{default}]: '
         else:
             prompt = PROMPT_PREFIX + text + ': '
         if USE_LIBEDIT:
@@ -138,7 +144,7 @@ def do_prompt(text: str, default: str = None, validator: Callable[[str], Any] =
             # sequence (see #5335).  To avoid the problem, all prompts are not colored
             # on libedit.
             pass
-        elif readline:
+        elif READLINE_AVAILABLE:
             # pass input_mode=True if readline available
             prompt = colorize(COLOR_QUESTION, prompt, input_mode=True)
         else:
@@ -156,8 +162,8 @@ def do_prompt(text: str, default: str = None, validator: Callable[[str], Any] =
 
 
 class QuickstartRenderer(SphinxRenderer):
-    def __init__(self, templatedir: str) -> None:
-        self.templatedir = templatedir or ''
+    def __init__(self, templatedir: str = '') -> None:
+        self.templatedir = templatedir
         super().__init__()
 
     def _has_custom_template(self, template_name: str) -> bool:
@@ -167,12 +173,9 @@ class QuickstartRenderer(SphinxRenderer):
               It will be removed in the future without deprecation period.
         """
         template = path.join(self.templatedir, path.basename(template_name))
-        if self.templatedir and path.exists(template):
-            return True
-        else:
-            return False
+        return bool(self.templatedir) and path.exists(template)
 
-    def render(self, template_name: str, context: Dict) -> str:
+    def render(self, template_name: str, context: dict[str, Any]) -> str:
         if self._has_custom_template(template_name):
             custom_template = path.join(self.templatedir, path.basename(template_name))
             return self.render_from_file(custom_template, context)
@@ -180,7 +183,7 @@ class QuickstartRenderer(SphinxRenderer):
             return super().render(template_name, context)
 
 
-def ask_user(d: Dict) -> None:
+def ask_user(d: dict[str, Any]) -> None:
     """Ask the user for quickstart values missing from *d*.
 
     Values are:
@@ -223,7 +226,7 @@ def ask_user(d: Dict) -> None:
         d['path'] = do_prompt(__('Please enter a new root path (or just Enter to exit)'),
                               '', is_path_or_empty)
         if not d['path']:
-            sys.exit(1)
+            raise SystemExit(1)
 
     if 'sep' not in d:
         print()
@@ -234,9 +237,9 @@ def ask_user(d: Dict) -> None:
 
     if 'dot' not in d:
         print()
-        print(__('Inside the root directory, two more directories will be created; "_templates"\n'      # NOQA
-                 'for custom HTML templates and "_static" for custom stylesheets and other static\n'    # NOQA
-                 'files. You can enter another prefix (such as ".") to replace the underscore.'))       # NOQA
+        print(__('Inside the root directory, two more directories will be created; "_templates"\n'      # noqa: E501
+                 'for custom HTML templates and "_static" for custom stylesheets and other static\n'    # noqa: E501
+                 'files. You can enter another prefix (such as ".") to replace the underscore.'))       # noqa: E501
         d['dot'] = do_prompt(__('Name prefix for templates and static dir'), '_', ok)
 
     if 'project' not in d:
@@ -251,7 +254,7 @@ def ask_user(d: Dict) -> None:
         print(__('Sphinx has the notion of a "version" and a "release" for the\n'
                  'software. Each version can have multiple releases. For example, for\n'
                  'Python the version is something like 2.5 or 3.0, while the release is\n'
-                 'something like 2.5.1 or 3.0a1. If you don\'t need this dual structure,\n'
+                 "something like 2.5.1 or 3.0a1. If you don't need this dual structure,\n"
                  'just set both to the same value.'))
         d['version'] = do_prompt(__('Project version'), '', allow_empty)
     if 'release' not in d:
@@ -259,12 +262,14 @@ def ask_user(d: Dict) -> None:
 
     if 'language' not in d:
         print()
-        print(__('If the documents are to be written in a language other than English,\n'
-                 'you can select a language here by its language code. Sphinx will then\n'
-                 'translate text that it generates into that language.\n'
-                 '\n'
-                 'For a list of supported codes, see\n'
-                 'https://www.sphinx-doc.org/en/master/usage/configuration.html#confval-language.'))  # NOQA
+        print(__(
+            'If the documents are to be written in a language other than English,\n'
+            'you can select a language here by its language code. Sphinx will then\n'
+            'translate text that it generates into that language.\n'
+            '\n'
+            'For a list of supported codes, see\n'
+            'https://www.sphinx-doc.org/en/master/usage/configuration.html#confval-language.',
+        ))
         d['language'] = do_prompt(__('Project language'), 'en')
         if d['language'] == 'en':
             d['language'] = None
@@ -297,7 +302,7 @@ def ask_user(d: Dict) -> None:
         print(__('Indicate which of the following Sphinx extensions should be enabled:'))
         d['extensions'] = []
         for name, description in EXTENSIONS.items():
-            if do_prompt('%s: %s (y/n)' % (name, description), 'n', boolean):
+            if do_prompt(f'{name}: {description} (y/n)', 'n', boolean):
                 d['extensions'].append('sphinx.ext.%s' % name)
 
         # Handle conflicting options
@@ -309,7 +314,7 @@ def ask_user(d: Dict) -> None:
     if 'makefile' not in d:
         print()
         print(__('A Makefile and a Windows command file can be generated for you so that you\n'
-                 'only have to run e.g. `make html\' instead of invoking sphinx-build\n'
+                 "only have to run e.g. `make html' instead of invoking sphinx-build\n"
                  'directly.'))
         d['makefile'] = do_prompt(__('Create Makefile? (y/n)'), 'y', boolean)
 
@@ -318,10 +323,11 @@ def ask_user(d: Dict) -> None:
     print()
 
 
-def generate(d: Dict, overwrite: bool = True, silent: bool = False, templatedir: str = None
-             ) -> None:
+def generate(
+    d: dict, overwrite: bool = True, silent: bool = False, templatedir: str | None = None,
+) -> None:
     """Generate project based on values in *d*."""
-    template = QuickstartRenderer(templatedir=templatedir)
+    template = QuickstartRenderer(templatedir or '')
 
     if 'mastertoctree' not in d:
         d['mastertoctree'] = ''
@@ -354,11 +360,11 @@ def generate(d: Dict, overwrite: bool = True, silent: bool = False, templatedir:
     ensuredir(path.join(srcdir, d['dot'] + 'templates'))
     ensuredir(path.join(srcdir, d['dot'] + 'static'))
 
-    def write_file(fpath: str, content: str, newline: str = None) -> None:
+    def write_file(fpath: str, content: str, newline: str | None = None) -> None:
         if overwrite or not path.isfile(fpath):
             if 'quiet' not in d:
                 print(__('Creating file %s.') % fpath)
-            with open(fpath, 'wt', encoding='utf-8', newline=newline) as f:
+            with open(fpath, 'w', encoding='utf-8', newline=newline) as f:
                 f.write(content)
         else:
             if 'quiet' not in d:
@@ -367,7 +373,7 @@ def generate(d: Dict, overwrite: bool = True, silent: bool = False, templatedir:
     conf_path = os.path.join(templatedir, 'conf.py_t') if templatedir else None
     if not conf_path or not path.isfile(conf_path):
         conf_path = os.path.join(package_dir, 'templates', 'quickstart', 'conf.py_t')
-    with open(conf_path) as f:
+    with open(conf_path, encoding="utf-8") as f:
         conf_text = f.read()
 
     write_file(path.join(srcdir, 'conf.py'), template.render_string(conf_text, d))
@@ -376,7 +382,7 @@ def generate(d: Dict, overwrite: bool = True, silent: bool = False, templatedir:
     if template._has_custom_template('quickstart/master_doc.rst_t'):
         msg = ('A custom template `master_doc.rst_t` found. It has been renamed to '
                '`root_doc.rst_t`.  Please rename it on your project too.')
-        print(colorize('red', msg))  # RemovedInSphinx60Warning
+        print(colorize('red', msg))
         write_file(masterfile, template.render('quickstart/master_doc.rst_t', d))
     else:
         write_file(masterfile, template.render('quickstart/root_doc.rst_t', d))
@@ -419,7 +425,7 @@ def generate(d: Dict, overwrite: bool = True, silent: bool = False, templatedir:
     print()
 
 
-def valid_dir(d: Dict) -> bool:
+def valid_dir(d: dict) -> bool:
     dir = d['path']
     if not path.exists(dir):
         return True
@@ -455,7 +461,7 @@ def get_parser() -> argparse.ArgumentParser:
         "\n"
         "sphinx-quickstart is an interactive tool that asks some questions about your\n"
         "project and then generates a complete documentation directory and sample\n"
-        "Makefile to be used with sphinx-build.\n"
+        "Makefile to be used with sphinx-build.\n",
     )
     parser = argparse.ArgumentParser(
         usage='%(prog)s [OPTIONS] <PROJECT_DIR>',
@@ -533,9 +539,9 @@ def get_parser() -> argparse.ArgumentParser:
     return parser
 
 
-def main(argv: List[str] = sys.argv[1:]) -> int:
-    sphinx.locale.setlocale(locale.LC_ALL, '')
-    sphinx.locale.init_console(os.path.join(package_dir, 'locale'), 'sphinx')
+def main(argv: list[str] = sys.argv[1:]) -> int:
+    locale.setlocale(locale.LC_ALL, '')
+    sphinx.locale.init_console()
 
     if not color_terminal():
         nocolor()
@@ -545,7 +551,7 @@ def main(argv: List[str] = sys.argv[1:]) -> int:
     try:
         args = parser.parse_args(argv)
     except SystemExit as err:
-        return err.code
+        return err.code  # type: ignore[return-value]
 
     d = vars(args)
     # delete None or False value
@@ -599,4 +605,4 @@ def main(argv: List[str] = sys.argv[1:]) -> int:
 
 
 if __name__ == '__main__':
-    sys.exit(main(sys.argv[1:]))
+    raise SystemExit(main())
