@@ -1,16 +1,9 @@
-"""
-    sphinx.domains.rst
-    ~~~~~~~~~~~~~~~~~~
+"""The reStructuredText domain."""
 
-    The reStructuredText domain.
-
-    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
-    :license: BSD, see LICENSE for details.
-"""
+from __future__ import annotations
 
 import re
-from typing import Any, Dict, Iterator, List, Tuple
-from typing import cast
+from typing import Any, Iterator, cast
 
 from docutils.nodes import Element
 from docutils.parsers.rst import directives
@@ -25,40 +18,76 @@ from sphinx.environment import BuildEnvironment
 from sphinx.locale import _, __
 from sphinx.roles import XRefRole
 from sphinx.util import logging
-from sphinx.util.nodes import make_refnode
-
+from sphinx.util.nodes import make_id, make_refnode
+from sphinx.util.typing import OptionSpec
 
 logger = logging.getLogger(__name__)
 
 dir_sig_re = re.compile(r'\.\. (.+?)::(.*)$')
 
 
-class ReSTMarkup(ObjectDescription):
+class ReSTMarkup(ObjectDescription[str]):
     """
     Description of generic reST markup.
     """
+    option_spec: OptionSpec = {
+        'noindex': directives.flag,
+        'noindexentry': directives.flag,
+        'nocontentsentry': directives.flag,
+    }
 
     def add_target_and_index(self, name: str, sig: str, signode: desc_signature) -> None:
-        targetname = self.objtype + '-' + name
-        if targetname not in self.state.document.ids:
-            signode['names'].append(targetname)
-            signode['ids'].append(targetname)
-            signode['first'] = (not self.names)
-            self.state.document.note_explicit_target(signode)
+        node_id = make_id(self.env, self.state.document, self.objtype, name)
+        signode['ids'].append(node_id)
+        self.state.document.note_explicit_target(signode)
 
-            domain = cast(ReSTDomain, self.env.get_domain('rst'))
-            domain.note_object(self.objtype, name, location=(self.env.docname, self.lineno))
+        domain = cast(ReSTDomain, self.env.get_domain('rst'))
+        domain.note_object(self.objtype, name, node_id, location=signode)
 
-        indextext = self.get_index_text(self.objtype, name)
-        if indextext:
-            self.indexnode['entries'].append(('single', indextext,
-                                              targetname, '', None))
+        if 'noindexentry' not in self.options:
+            indextext = self.get_index_text(self.objtype, name)
+            if indextext:
+                self.indexnode['entries'].append(('single', indextext, node_id, '', None))
 
     def get_index_text(self, objectname: str, name: str) -> str:
         return ''
 
+    def make_old_id(self, name: str) -> str:
+        """Generate old styled node_id for reST markups.
 
-def parse_directive(d: str) -> Tuple[str, str]:
+        .. note:: Old Styled node_id was used until Sphinx-3.0.
+                  This will be removed in Sphinx-5.0.
+        """
+        return self.objtype + '-' + name
+
+    def _object_hierarchy_parts(self, sig_node: desc_signature) -> tuple[str, ...]:
+        if 'fullname' not in sig_node:
+            return ()
+        directive_names = []
+        for parent in self.env.ref_context.get('rst:directives', ()):
+            directive_names += parent.split(':')
+        name = sig_node['fullname']
+        return tuple(directive_names + name.split(':'))
+
+    def _toc_entry_name(self, sig_node: desc_signature) -> str:
+        if not sig_node.get('_toc_parts'):
+            return ''
+
+        config = self.env.app.config
+        objtype = sig_node.parent.get('objtype')
+        *parents, name = sig_node['_toc_parts']
+        if objtype == 'directive:option':
+            return f':{name}:'
+        if config.toc_object_entries_show_parents in {'domain', 'all'}:
+            name = ':'.join(sig_node['_toc_parts'])
+        if objtype == 'role':
+            return f':{name}:'
+        if objtype == 'directive':
+            return f'.. {name}::'
+        return ''
+
+
+def parse_directive(d: str) -> tuple[str, str]:
     """Parse a directive signature.
 
     Returns (directive, arguments) string tuple.  If no arguments are given,
@@ -84,7 +113,8 @@ class ReSTDirective(ReSTMarkup):
     """
     def handle_signature(self, sig: str, signode: desc_signature) -> str:
         name, args = parse_directive(sig)
-        desc_name = '.. %s::' % name
+        desc_name = f'.. {name}::'
+        signode['fullname'] = name.strip()
         signode += addnodes.desc_name(desc_name, desc_name)
         if len(args) > 0:
             signode += addnodes.desc_addname(args, args)
@@ -108,7 +138,7 @@ class ReSTDirectiveOption(ReSTMarkup):
     """
     Description of an option for reST directive.
     """
-    option_spec = ReSTMarkup.option_spec.copy()
+    option_spec: OptionSpec = ReSTMarkup.option_spec.copy()
     option_spec.update({
         'type': directives.unchanged,
     })
@@ -119,7 +149,9 @@ class ReSTDirectiveOption(ReSTMarkup):
         except ValueError:
             name, argument = sig, None
 
-        signode += addnodes.desc_name(':%s:' % name, ':%s:' % name)
+        desc_name = f':{name}:'
+        signode['fullname'] = name.strip()
+        signode += addnodes.desc_name(desc_name, desc_name)
         if argument:
             signode += addnodes.desc_annotation(' ' + argument, ' ' + argument)
         if self.options.get('type'):
@@ -128,27 +160,30 @@ class ReSTDirectiveOption(ReSTMarkup):
         return name
 
     def add_target_and_index(self, name: str, sig: str, signode: desc_signature) -> None:
-        directive_name = self.current_directive
-        targetname = '-'.join([self.objtype, self.current_directive, name])
-        if targetname not in self.state.document.ids:
-            signode['names'].append(targetname)
-            signode['ids'].append(targetname)
-            signode['first'] = (not self.names)
-            self.state.document.note_explicit_target(signode)
+        domain = cast(ReSTDomain, self.env.get_domain('rst'))
 
-            objname = ':'.join(filter(None, [directive_name, name]))
-            domain = cast(ReSTDomain, self.env.get_domain('rst'))
-            domain.note_object(self.objtype, objname, location=(self.env.docname, self.lineno))
+        directive_name = self.current_directive
+        if directive_name:
+            prefix = '-'.join([self.objtype, directive_name])
+            objname = ':'.join([directive_name, name])
+        else:
+            prefix = self.objtype
+            objname = name
+
+        node_id = make_id(self.env, self.state.document, prefix, name)
+        signode['ids'].append(node_id)
+        self.state.document.note_explicit_target(signode)
+        domain.note_object(self.objtype, objname, node_id, location=signode)
 
         if directive_name:
             key = name[0].upper()
             pair = [_('%s (directive)') % directive_name,
                     _(':%s: (directive option)') % name]
-            self.indexnode['entries'].append(('pair', '; '.join(pair), targetname, '', key))
+            self.indexnode['entries'].append(('pair', '; '.join(pair), node_id, '', key))
         else:
             key = name[0].upper()
             text = _(':%s: (directive option)') % name
-            self.indexnode['entries'].append(('single', text, targetname, '', key))
+            self.indexnode['entries'].append(('single', text, node_id, '', key))
 
     @property
     def current_directive(self) -> str:
@@ -158,13 +193,23 @@ class ReSTDirectiveOption(ReSTMarkup):
         else:
             return ''
 
+    def make_old_id(self, name: str) -> str:
+        """Generate old styled node_id for directive options.
+
+        .. note:: Old Styled node_id was used until Sphinx-3.0.
+                  This will be removed in Sphinx-5.0.
+        """
+        return '-'.join([self.objtype, self.current_directive, name])
+
 
 class ReSTRole(ReSTMarkup):
     """
     Description of a reST role.
     """
     def handle_signature(self, sig: str, signode: desc_signature) -> str:
-        signode += addnodes.desc_name(':%s:' % sig, ':%s:' % sig)
+        desc_name = f':{sig}:'
+        signode['fullname'] = sig.strip()
+        signode += addnodes.desc_name(desc_name, desc_name)
         return sig
 
     def get_index_text(self, objectname: str, name: str) -> str:
@@ -190,69 +235,69 @@ class ReSTDomain(Domain):
         'dir':  XRefRole(),
         'role': XRefRole(),
     }
-    initial_data = {
+    initial_data: dict[str, dict[tuple[str, str], str]] = {
         'objects': {},  # fullname -> docname, objtype
-    }  # type: Dict[str, Dict[Tuple[str, str], str]]
+    }
 
     @property
-    def objects(self) -> Dict[Tuple[str, str], str]:
-        return self.data.setdefault('objects', {})  # (objtype, fullname) -> docname
+    def objects(self) -> dict[tuple[str, str], tuple[str, str]]:
+        return self.data.setdefault('objects', {})  # (objtype, fullname) -> (docname, node_id)
 
-    def note_object(self, objtype: str, name: str, location: Any = None) -> None:
+    def note_object(self, objtype: str, name: str, node_id: str, location: Any = None) -> None:
         if (objtype, name) in self.objects:
-            docname = self.objects[objtype, name]
+            docname, node_id = self.objects[objtype, name]
             logger.warning(__('duplicate description of %s %s, other instance in %s') %
                            (objtype, name, docname), location=location)
 
-        self.objects[objtype, name] = self.env.docname
+        self.objects[objtype, name] = (self.env.docname, node_id)
 
     def clear_doc(self, docname: str) -> None:
-        for (typ, name), doc in list(self.objects.items()):
+        for (typ, name), (doc, _node_id) in list(self.objects.items()):
             if doc == docname:
                 del self.objects[typ, name]
 
-    def merge_domaindata(self, docnames: List[str], otherdata: Dict) -> None:
+    def merge_domaindata(self, docnames: list[str], otherdata: dict[str, Any]) -> None:
         # XXX check duplicates
-        for (typ, name), doc in otherdata['objects'].items():
+        for (typ, name), (doc, node_id) in otherdata['objects'].items():
             if doc in docnames:
-                self.objects[typ, name] = doc
+                self.objects[typ, name] = (doc, node_id)
 
     def resolve_xref(self, env: BuildEnvironment, fromdocname: str, builder: Builder,
-                     typ: str, target: str, node: pending_xref, contnode: Element
-                     ) -> Element:
+                     typ: str, target: str, node: pending_xref, contnode: Element,
+                     ) -> Element | None:
         objtypes = self.objtypes_for_role(typ)
         for objtype in objtypes:
-            todocname = self.objects.get((objtype, target))
-            if todocname:
-                return make_refnode(builder, fromdocname, todocname,
-                                    objtype + '-' + target,
+            result = self.objects.get((objtype, target))
+            if result:
+                todocname, node_id = result
+                return make_refnode(builder, fromdocname, todocname, node_id,
                                     contnode, target + ' ' + objtype)
         return None
 
     def resolve_any_xref(self, env: BuildEnvironment, fromdocname: str, builder: Builder,
-                         target: str, node: pending_xref, contnode: Element
-                         ) -> List[Tuple[str, Element]]:
-        results = []  # type: List[Tuple[str, Element]]
+                         target: str, node: pending_xref, contnode: Element,
+                         ) -> list[tuple[str, Element]]:
+        results: list[tuple[str, Element]] = []
         for objtype in self.object_types:
-            todocname = self.objects.get((objtype, target))
-            if todocname:
+            result = self.objects.get((objtype, target))
+            if result:
+                todocname, node_id = result
                 results.append(('rst:' + self.role_for_objtype(objtype),
-                                make_refnode(builder, fromdocname, todocname,
-                                             objtype + '-' + target,
+                                make_refnode(builder, fromdocname, todocname, node_id,
                                              contnode, target + ' ' + objtype)))
         return results
 
-    def get_objects(self) -> Iterator[Tuple[str, str, str, str, str, int]]:
-        for (typ, name), docname in self.data['objects'].items():
-            yield name, name, typ, docname, typ + '-' + name, 1
+    def get_objects(self) -> Iterator[tuple[str, str, str, str, str, int]]:
+        for (typ, name), (docname, node_id) in self.data['objects'].items():
+            yield name, name, typ, docname, node_id, 1
 
 
-def setup(app: Sphinx) -> Dict[str, Any]:
+def setup(app: Sphinx) -> dict[str, Any]:
     app.add_domain(ReSTDomain)
 
     return {
         'version': 'builtin',
-        'env_version': 1,
+        'env_version': 2,
         'parallel_read_safe': True,
         'parallel_write_safe': True,
     }
