@@ -5,7 +5,9 @@ The todolist directive collects all todos of your project and lists them along
 with a backlink to the original location.
 """
 
-from typing import Any, Dict, List, Tuple, cast
+from __future__ import annotations
+
+from typing import Any, cast
 
 from docutils import nodes
 from docutils.nodes import Element, Node
@@ -22,7 +24,7 @@ from sphinx.locale import _, __
 from sphinx.util import logging, texescape
 from sphinx.util.docutils import SphinxDirective, new_document
 from sphinx.util.typing import OptionSpec
-from sphinx.writers.html import HTMLTranslator
+from sphinx.writers.html import HTML5Translator
 from sphinx.writers.latex import LaTeXTranslator
 
 logger = logging.getLogger(__name__)
@@ -51,11 +53,11 @@ class Todo(BaseAdmonition, SphinxDirective):
         'name': directives.unchanged,
     }
 
-    def run(self) -> List[Node]:
+    def run(self) -> list[Node]:
         if not self.options.get('class'):
             self.options['class'] = ['admonition-todo']
 
-        (todo,) = super().run()  # type: Tuple[Node]
+        (todo,) = super().run()
         if isinstance(todo, nodes.system_message):
             return [todo]
         elif isinstance(todo, todo_node):
@@ -74,13 +76,13 @@ class TodoDomain(Domain):
     label = 'todo'
 
     @property
-    def todos(self) -> Dict[str, List[todo_node]]:
+    def todos(self) -> dict[str, list[todo_node]]:
         return self.data.setdefault('todos', {})
 
     def clear_doc(self, docname: str) -> None:
         self.todos.pop(docname, None)
 
-    def merge_domaindata(self, docnames: List[str], otherdata: Dict) -> None:
+    def merge_domaindata(self, docnames: list[str], otherdata: dict) -> None:
         for docname in docnames:
             self.todos[docname] = otherdata['todos'][docname]
 
@@ -107,7 +109,7 @@ class TodoList(SphinxDirective):
     final_argument_whitespace = False
     option_spec: OptionSpec = {}
 
-    def run(self) -> List[Node]:
+    def run(self) -> list[Node]:
         # Simply insert an empty todolist node which will be replaced later
         # when process_todo_nodes is called
         return [todolist('')]
@@ -124,14 +126,14 @@ class TodoListProcessor:
         self.process(doctree, docname)
 
     def process(self, doctree: nodes.document, docname: str) -> None:
-        todos: List[todo_node] = sum(self.domain.todos.values(), [])
+        todos: list[todo_node] = sum(self.domain.todos.values(), [])
         for node in list(doctree.findall(todolist)):
             if not self.config.todo_include_todos:
                 node.parent.remove(node)
                 continue
 
             if node.get('ids'):
-                content: List[Element] = [nodes.target()]
+                content: list[Element] = [nodes.target()]
             else:
                 content = []
 
@@ -188,14 +190,14 @@ class TodoListProcessor:
         self.document.remove(todo)
 
 
-def visit_todo_node(self: HTMLTranslator, node: todo_node) -> None:
+def visit_todo_node(self: HTML5Translator, node: todo_node) -> None:
     if self.config.todo_include_todos:
         self.visit_admonition(node)
     else:
         raise nodes.SkipNode
 
 
-def depart_todo_node(self: HTMLTranslator, node: todo_node) -> None:
+def depart_todo_node(self: HTML5Translator, node: todo_node) -> None:
     self.depart_admonition(node)
 
 
@@ -216,7 +218,7 @@ def latex_depart_todo_node(self: LaTeXTranslator, node: todo_node) -> None:
     self.body.append('\\end{sphinxadmonition}\n')
 
 
-def setup(app: Sphinx) -> Dict[str, Any]:
+def setup(app: Sphinx) -> dict[str, Any]:
     app.add_event('todo-defined')
     app.add_config_value('todo_include_todos', False, 'html')
     app.add_config_value('todo_link_only', False, 'html')
@@ -237,5 +239,5 @@ def setup(app: Sphinx) -> Dict[str, Any]:
     return {
         'version': sphinx.__display_version__,
         'env_version': 2,
-        'parallel_read_safe': True
+        'parallel_read_safe': True,
     }
