@@ -1,28 +1,29 @@
 """Sphinx component registry."""
 
+from __future__ import annotations
+
+import sys
 import traceback
 import warnings
 from importlib import import_module
 from types import MethodType
-from typing import (TYPE_CHECKING, Any, Callable, Dict, Iterator, List, Optional, Tuple, Type,
-                    Union)
+from typing import TYPE_CHECKING, Any, Callable, Iterator
 
 from docutils import nodes
 from docutils.core import Publisher
-from docutils.io import Input
 from docutils.nodes import Element, Node, TextElement
 from docutils.parsers import Parser
 from docutils.parsers.rst import Directive
 from docutils.transforms import Transform
 
-try:  # Python < 3.10 (backport)
-    from importlib_metadata import entry_points
-except ImportError:
+if sys.version_info >= (3, 10):
     from importlib.metadata import entry_points
+else:
+    from importlib_metadata import entry_points
 
 from sphinx.builders import Builder
 from sphinx.config import Config
-from sphinx.deprecation import RemovedInSphinx60Warning, RemovedInSphinx70Warning
+from sphinx.deprecation import RemovedInSphinx70Warning
 from sphinx.domains import Domain, Index, ObjType
 from sphinx.domains.std import GenericObject, Target
 from sphinx.environment import BuildEnvironment
@@ -45,92 +46,89 @@ logger = logging.getLogger(__name__)
 # list of deprecated extensions. Keys are extension name.
 # Values are Sphinx version that merge the extension.
 EXTENSION_BLACKLIST = {
-    "sphinxjp.themecore": "1.2"
+    "sphinxjp.themecore": "1.2",
 }
 
 
 class SphinxComponentRegistry:
     def __init__(self) -> None:
         #: special attrgetter for autodoc; class object -> attrgetter
-        self.autodoc_attrgettrs: Dict[Type, Callable[[Any, str, Any], Any]] = {}
+        self.autodoc_attrgettrs: dict[type, Callable[[Any, str, Any], Any]] = {}
 
         #: builders; a dict of builder name -> bulider class
-        self.builders: Dict[str, Type[Builder]] = {}
+        self.builders: dict[str, type[Builder]] = {}
 
         #: autodoc documenters; a dict of documenter name -> documenter class
-        self.documenters: Dict[str, Type[Documenter]] = {}
+        self.documenters: dict[str, type[Documenter]] = {}
 
         #: css_files; a list of tuple of filename and attributes
-        self.css_files: List[Tuple[str, Dict[str, Any]]] = []
+        self.css_files: list[tuple[str, dict[str, Any]]] = []
 
         #: domains; a dict of domain name -> domain class
-        self.domains: Dict[str, Type[Domain]] = {}
+        self.domains: dict[str, type[Domain]] = {}
 
         #: additional directives for domains
         #: a dict of domain name -> dict of directive name -> directive
-        self.domain_directives: Dict[str, Dict[str, Any]] = {}
+        self.domain_directives: dict[str, dict[str, type[Directive]]] = {}
 
         #: additional indices for domains
         #: a dict of domain name -> list of index class
-        self.domain_indices: Dict[str, List[Type[Index]]] = {}
+        self.domain_indices: dict[str, list[type[Index]]] = {}
 
         #: additional object types for domains
         #: a dict of domain name -> dict of objtype name -> objtype
-        self.domain_object_types: Dict[str, Dict[str, ObjType]] = {}
+        self.domain_object_types: dict[str, dict[str, ObjType]] = {}
 
         #: additional roles for domains
         #: a dict of domain name -> dict of role name -> role impl.
-        self.domain_roles: Dict[str, Dict[str, Union[RoleFunction, XRefRole]]] = {}
+        self.domain_roles: dict[str, dict[str, RoleFunction | XRefRole]] = {}
 
         #: additional enumerable nodes
         #: a dict of node class -> tuple of figtype and title_getter function
-        self.enumerable_nodes: Dict[Type[Node], Tuple[str, TitleGetter]] = {}
+        self.enumerable_nodes: dict[type[Node], tuple[str, TitleGetter]] = {}
 
         #: HTML inline and block math renderers
         #: a dict of name -> tuple of visit function and depart function
-        self.html_inline_math_renderers: Dict[str, Tuple[Callable, Callable]] = {}
-        self.html_block_math_renderers: Dict[str, Tuple[Callable, Callable]] = {}
+        self.html_inline_math_renderers: dict[str, tuple[Callable, Callable]] = {}
+        self.html_block_math_renderers: dict[str, tuple[Callable, Callable]] = {}
 
         #: HTML assets
         self.html_assets_policy: str = 'per_page'
 
         #: HTML themes
-        self.html_themes: Dict[str, str] = {}
+        self.html_themes: dict[str, str] = {}
 
         #: js_files; list of JS paths or URLs
-        self.js_files: List[Tuple[str, Dict[str, Any]]] = []
+        self.js_files: list[tuple[str, dict[str, Any]]] = []
 
         #: LaTeX packages; list of package names and its options
-        self.latex_packages: List[Tuple[str, str]] = []
+        self.latex_packages: list[tuple[str, str]] = []
 
-        self.latex_packages_after_hyperref: List[Tuple[str, str]] = []
+        self.latex_packages_after_hyperref: list[tuple[str, str]] = []
 
         #: post transforms; list of transforms
-        self.post_transforms: List[Type[Transform]] = []
+        self.post_transforms: list[type[Transform]] = []
 
         #: source paresrs; file type -> parser class
-        self.source_parsers: Dict[str, Type[Parser]] = {}
-
-        #: source inputs; file type -> input class
-        self.source_inputs: Dict[str, Type[Input]] = {}
+        self.source_parsers: dict[str, type[Parser]] = {}
 
         #: source suffix: suffix -> file type
-        self.source_suffix: Dict[str, str] = {}
+        self.source_suffix: dict[str, str] = {}
 
         #: custom translators; builder name -> translator class
-        self.translators: Dict[str, Type[nodes.NodeVisitor]] = {}
+        self.translators: dict[str, type[nodes.NodeVisitor]] = {}
 
         #: custom handlers for translators
         #: a dict of builder name -> dict of node name -> visitor and departure functions
-        self.translation_handlers: Dict[str, Dict[str, Tuple[Callable, Callable]]] = {}
+        self.translation_handlers: dict[str, dict[str, tuple[Callable, Callable]]] = {}
 
         #: additional transforms; list of transforms
-        self.transforms: List[Type[Transform]] = []
+        self.transforms: list[type[Transform]] = []
 
         # private cache of Docutils Publishers (file type -> publisher object)
-        self.publishers: Dict[str, Publisher] = {}
+        self.publishers: dict[str, Publisher] = {}
 
-    def add_builder(self, builder: Type[Builder], override: bool = False) -> None:
+    def add_builder(self, builder: type[Builder], override: bool = False) -> None:
         logger.debug('[app] adding builder: %r', builder)
         if not hasattr(builder, 'name'):
             raise ExtensionError(__('Builder class %s has no "name" attribute') % builder)
@@ -139,7 +137,7 @@ class SphinxComponentRegistry:
                                  (builder.name, self.builders[builder.name].__module__))
         self.builders[builder.name] = builder
 
-    def preload_builder(self, app: "Sphinx", name: str) -> None:
+    def preload_builder(self, app: Sphinx, name: str) -> None:
         if name is None:
             return
 
@@ -153,8 +151,8 @@ class SphinxComponentRegistry:
 
             self.load_extension(app, entry_point.module)
 
-    def create_builder(self, app: "Sphinx", name: str,
-                       env: BuildEnvironment = None) -> Builder:
+    def create_builder(self, app: Sphinx, name: str,
+                       env: BuildEnvironment | None = None) -> Builder:
         if name not in self.builders:
             raise SphinxError(__('Builder name %s not registered') % name)
 
@@ -171,7 +169,7 @@ class SphinxComponentRegistry:
                 builder.set_environment(env)
             return builder
 
-    def add_domain(self, domain: Type[Domain], override: bool = False) -> None:
+    def add_domain(self, domain: type[Domain], override: bool = False) -> None:
         logger.debug('[app] adding domain: %r', domain)
         if domain.name in self.domains and not override:
             raise ExtensionError(__('domain %s already registered') % domain.name)
@@ -194,19 +192,19 @@ class SphinxComponentRegistry:
             yield domain
 
     def add_directive_to_domain(self, domain: str, name: str,
-                                cls: Type[Directive], override: bool = False) -> None:
+                                cls: type[Directive], override: bool = False) -> None:
         logger.debug('[app] adding directive to domain: %r', (domain, name, cls))
         if domain not in self.domains:
             raise ExtensionError(__('domain %s not yet registered') % domain)
 
-        directives = self.domain_directives.setdefault(domain, {})
+        directives: dict[str, type[Directive]] = self.domain_directives.setdefault(domain, {})
         if name in directives and not override:
             raise ExtensionError(__('The %r directive is already registered to domain %s') %
                                  (name, domain))
         directives[name] = cls
 
     def add_role_to_domain(self, domain: str, name: str,
-                           role: Union[RoleFunction, XRefRole], override: bool = False
+                           role: RoleFunction | XRefRole, override: bool = False,
                            ) -> None:
         logger.debug('[app] adding role to domain: %r', (domain, name, role))
         if domain not in self.domains:
@@ -217,7 +215,7 @@ class SphinxComponentRegistry:
                                  (name, domain))
         roles[name] = role
 
-    def add_index_to_domain(self, domain: str, index: Type[Index],
+    def add_index_to_domain(self, domain: str, index: type[Index],
                             override: bool = False) -> None:
         logger.debug('[app] adding index to domain: %r', (domain, index))
         if domain not in self.domains:
@@ -228,10 +226,17 @@ class SphinxComponentRegistry:
                                  (index.name, domain))
         indices.append(index)
 
-    def add_object_type(self, directivename: str, rolename: str, indextemplate: str = '',
-                        parse_node: Callable = None, ref_nodeclass: Type[TextElement] = None,
-                        objname: str = '', doc_field_types: List = [], override: bool = False
-                        ) -> None:
+    def add_object_type(
+        self,
+        directivename: str,
+        rolename: str,
+        indextemplate: str = '',
+        parse_node: Callable | None = None,
+        ref_nodeclass: type[TextElement] | None = None,
+        objname: str = '',
+        doc_field_types: list = [],
+        override: bool = False,
+    ) -> None:
         logger.debug('[app] adding object type: %r',
                      (directivename, rolename, indextemplate, parse_node,
                       ref_nodeclass, objname, doc_field_types))
@@ -252,9 +257,15 @@ class SphinxComponentRegistry:
                                  directivename)
         object_types[directivename] = ObjType(objname or directivename, rolename)
 
-    def add_crossref_type(self, directivename: str, rolename: str, indextemplate: str = '',
-                          ref_nodeclass: Type[TextElement] = None, objname: str = '',
-                          override: bool = False) -> None:
+    def add_crossref_type(
+        self,
+        directivename: str,
+        rolename: str,
+        indextemplate: str = '',
+        ref_nodeclass: type[TextElement] | None = None,
+        objname: str = '',
+        override: bool = False,
+    ) -> None:
         logger.debug('[app] adding crossref type: %r',
                      (directivename, rolename, indextemplate, ref_nodeclass, objname))
 
@@ -276,10 +287,9 @@ class SphinxComponentRegistry:
         logger.debug('[app] adding source_suffix: %r, %r', suffix, filetype)
         if suffix in self.source_suffix and not override:
             raise ExtensionError(__('source_suffix %r is already registered') % suffix)
-        else:
-            self.source_suffix[suffix] = filetype
+        self.source_suffix[suffix] = filetype
 
-    def add_source_parser(self, parser: Type[Parser], override: bool = False) -> None:
+    def add_source_parser(self, parser: type[Parser], override: bool = False) -> None:
         logger.debug('[app] adding search source_parser: %r', parser)
 
         # create a map from filetype to parser
@@ -287,47 +297,33 @@ class SphinxComponentRegistry:
             if filetype in self.source_parsers and not override:
                 raise ExtensionError(__('source_parser for %r is already registered') %
                                      filetype)
-            else:
-                self.source_parsers[filetype] = parser
+            self.source_parsers[filetype] = parser
 
-    def get_source_parser(self, filetype: str) -> Type[Parser]:
+    def get_source_parser(self, filetype: str) -> type[Parser]:
         try:
             return self.source_parsers[filetype]
         except KeyError as exc:
             raise SphinxError(__('Source parser for %s not registered') % filetype) from exc
 
-    def get_source_parsers(self) -> Dict[str, Type[Parser]]:
+    def get_source_parsers(self) -> dict[str, type[Parser]]:
         return self.source_parsers
 
-    def create_source_parser(self, app: "Sphinx", filename: str) -> Parser:
+    def create_source_parser(self, app: Sphinx, filename: str) -> Parser:
         parser_class = self.get_source_parser(filename)
         parser = parser_class()
         if isinstance(parser, SphinxParser):
             parser.set_application(app)
         return parser
 
-    def get_source_input(self, filetype: str) -> Optional[Type[Input]]:
-        warnings.warn('SphinxComponentRegistry.get_source_input() is deprecated.',
-                      RemovedInSphinx60Warning)
-
-        try:
-            return self.source_inputs[filetype]
-        except KeyError:
-            try:
-                # use special source_input for unknown filetype
-                return self.source_inputs['*']
-            except KeyError:
-                return None
-
-    def add_translator(self, name: str, translator: Type[nodes.NodeVisitor],
+    def add_translator(self, name: str, translator: type[nodes.NodeVisitor],
                        override: bool = False) -> None:
         logger.debug('[app] Change of translator for the %s builder.', name)
         if name in self.translators and not override:
             raise ExtensionError(__('Translator for %r already exists') % name)
         self.translators[name] = translator
 
-    def add_translation_handlers(self, node: Type[Element],
-                                 **kwargs: Tuple[Callable, Callable]) -> None:
+    def add_translation_handlers(self, node: type[Element],
+                                 **kwargs: tuple[Callable, Callable]) -> None:
         logger.debug('[app] adding translation_handlers: %r, %r', node, kwargs)
         for builder_name, handlers in kwargs.items():
             translation_handlers = self.translation_handlers.setdefault(builder_name, {})
@@ -337,10 +333,10 @@ class SphinxComponentRegistry:
             except ValueError as exc:
                 raise ExtensionError(
                     __('kwargs for add_node() must be a (visit, depart) '
-                       'function tuple: %r=%r') % (builder_name, handlers)
+                       'function tuple: %r=%r') % (builder_name, handlers),
                 ) from exc
 
-    def get_translator_class(self, builder: Builder) -> Type[nodes.NodeVisitor]:
+    def get_translator_class(self, builder: Builder) -> type[nodes.NodeVisitor]:
         return self.translators.get(builder.name,
                                     builder.default_translator_class)
 
@@ -362,24 +358,24 @@ class SphinxComponentRegistry:
 
         return translator
 
-    def add_transform(self, transform: Type[Transform]) -> None:
+    def add_transform(self, transform: type[Transform]) -> None:
         logger.debug('[app] adding transform: %r', transform)
         self.transforms.append(transform)
 
-    def get_transforms(self) -> List[Type[Transform]]:
+    def get_transforms(self) -> list[type[Transform]]:
         return self.transforms
 
-    def add_post_transform(self, transform: Type[Transform]) -> None:
+    def add_post_transform(self, transform: type[Transform]) -> None:
         logger.debug('[app] adding post transform: %r', transform)
         self.post_transforms.append(transform)
 
-    def get_post_transforms(self) -> List[Type[Transform]]:
+    def get_post_transforms(self) -> list[type[Transform]]:
         return self.post_transforms
 
-    def add_documenter(self, objtype: str, documenter: Type["Documenter"]) -> None:
+    def add_documenter(self, objtype: str, documenter: type[Documenter]) -> None:
         self.documenters[objtype] = documenter
 
-    def add_autodoc_attrgetter(self, typ: Type,
+    def add_autodoc_attrgetter(self, typ: type,
                                attrgetter: Callable[[Any, str, Any], Any]) -> None:
         self.autodoc_attrgettrs[typ] = attrgetter
 
@@ -404,16 +400,20 @@ class SphinxComponentRegistry:
         else:
             self.latex_packages.append((name, options))
 
-    def add_enumerable_node(self, node: Type[Node], figtype: str,
-                            title_getter: TitleGetter = None, override: bool = False) -> None:
+    def add_enumerable_node(
+        self,
+        node: type[Node],
+        figtype: str,
+        title_getter: TitleGetter | None = None, override: bool = False,
+    ) -> None:
         logger.debug('[app] adding enumerable node: (%r, %r, %r)', node, figtype, title_getter)
         if node in self.enumerable_nodes and not override:
             raise ExtensionError(__('enumerable_node %r already registered') % node)
         self.enumerable_nodes[node] = (figtype, title_getter)
 
     def add_html_math_renderer(self, name: str,
-                               inline_renderers: Tuple[Callable, Callable],
-                               block_renderers: Tuple[Callable, Callable]) -> None:
+                               inline_renderers: tuple[Callable, Callable],
+                               block_renderers: tuple[Callable, Callable]) -> None:
         logger.debug('[app] adding html_math_renderer: %s, %r, %r',
                      name, inline_renderers, block_renderers)
         if name in self.html_inline_math_renderers:
@@ -425,7 +425,7 @@ class SphinxComponentRegistry:
     def add_html_theme(self, name: str, theme_path: str) -> None:
         self.html_themes[name] = theme_path
 
-    def load_extension(self, app: "Sphinx", extname: str) -> None:
+    def load_extension(self, app: Sphinx, extname: str) -> None:
         """Load a Sphinx extension."""
         if extname in app.extensions:  # already loaded
             return
@@ -449,7 +449,7 @@ class SphinxComponentRegistry:
             if setup is None:
                 logger.warning(__('extension %r has no setup() function; is it really '
                                   'a Sphinx extension module?'), extname)
-                metadata: Dict[str, Any] = {}
+                metadata: dict[str, Any] = {}
             else:
                 try:
                     metadata = setup(app)
@@ -458,7 +458,7 @@ class SphinxComponentRegistry:
                     raise VersionRequirementError(
                         __('The %s extension used by this project needs at least '
                            'Sphinx v%s; it therefore cannot be built with this '
-                           'version.') % (extname, err)
+                           'version.') % (extname, err),
                     ) from err
 
             if metadata is None:
@@ -471,14 +471,14 @@ class SphinxComponentRegistry:
 
             app.extensions[extname] = Extension(extname, mod, **metadata)
 
-    def get_envversion(self, app: "Sphinx") -> Dict[str, str]:
+    def get_envversion(self, app: Sphinx) -> dict[str, str]:
         from sphinx.environment import ENV_VERSION
         envversion = {ext.name: ext.metadata['env_version'] for ext in app.extensions.values()
                       if ext.metadata.get('env_version')}
         envversion['sphinx'] = ENV_VERSION
         return envversion
 
-    def get_publisher(self, app: "Sphinx", filetype: str) -> Publisher:
+    def get_publisher(self, app: Sphinx, filetype: str) -> Publisher:
         try:
             return self.publishers[filetype]
         except KeyError:
@@ -488,10 +488,10 @@ class SphinxComponentRegistry:
         return publisher
 
 
-def merge_source_suffix(app: "Sphinx", config: Config) -> None:
+def merge_source_suffix(app: Sphinx, config: Config) -> None:
     """Merge any user-specified source_suffix with any added by extensions."""
     for suffix, filetype in app.registry.source_suffix.items():
-        if suffix not in app.config.source_suffix:
+        if suffix not in app.config.source_suffix:  # NoQA: SIM114
             app.config.source_suffix[suffix] = filetype
         elif app.config.source_suffix[suffix] is None:
             # filetype is not specified (default filetype).
@@ -502,7 +502,7 @@ def merge_source_suffix(app: "Sphinx", config: Config) -> None:
     app.registry.source_suffix = app.config.source_suffix
 
 
-def setup(app: "Sphinx") -> Dict[str, Any]:
+def setup(app: Sphinx) -> dict[str, Any]:
     app.connect('config-inited', merge_source_suffix, priority=800)
 
     return {
