@@ -1,14 +1,14 @@
 """Utility functions for docutils."""
 
+from __future__ import annotations
+
 import os
 import re
 import warnings
 from contextlib import contextmanager
 from copy import copy
 from os import path
-from types import ModuleType
-from typing import (IO, TYPE_CHECKING, Any, Callable, Dict, Generator, List, Optional, Set,
-                    Tuple, Type, cast)
+from typing import IO, TYPE_CHECKING, Any, Callable, Generator, cast
 
 import docutils
 from docutils import nodes
@@ -18,8 +18,8 @@ from docutils.parsers.rst import Directive, directives, roles
 from docutils.parsers.rst.states import Inliner
 from docutils.statemachine import State, StateMachine, StringList
 from docutils.utils import Reporter, unescape
+from docutils.writers._html_base import HTMLTranslator
 
-from sphinx.deprecation import RemovedInSphinx70Warning, deprecated_alias
 from sphinx.errors import SphinxError
 from sphinx.locale import _, __
 from sphinx.util import logging
@@ -29,19 +29,32 @@ logger = logging.getLogger(__name__)
 report_re = re.compile('^(.+?:(?:\\d+)?): \\((DEBUG|INFO|WARNING|ERROR|SEVERE)/(\\d+)?\\) ')
 
 if TYPE_CHECKING:
+    from types import ModuleType
+
+    from docutils.frontend import Values
+
     from sphinx.builders import Builder
     from sphinx.config import Config
     from sphinx.environment import BuildEnvironment
 
-deprecated_alias('sphinx.util.docutils',
-                 {
-                     '__version_info__': docutils.__version_info__,
-                 },
-                 RemovedInSphinx70Warning,
-                 {
-                     '__version_info__': 'docutils.__version_info__',
-                 })
-additional_nodes: Set[Type[Element]] = set()
+# deprecated name -> (object to return, canonical path or empty string)
+_DEPRECATED_OBJECTS = {
+    '__version_info__': (docutils.__version_info__, 'docutils.__version_info__'),
+}
+
+
+def __getattr__(name):
+    if name not in _DEPRECATED_OBJECTS:
+        raise AttributeError(f'module {__name__!r} has no attribute {name!r}')
+
+    from sphinx.deprecation import _deprecation_warning
+
+    deprecated_object, canonical_name = _DEPRECATED_OBJECTS[name]
+    _deprecation_warning(__name__, name, canonical_name, remove=(7, 0))
+    return deprecated_object
+
+
+additional_nodes: set[type[Element]] = set()
 
 
 @contextmanager
@@ -66,7 +79,7 @@ def is_directive_registered(name: str) -> bool:
     return name in directives._directives  # type: ignore
 
 
-def register_directive(name: str, directive: Type[Directive]) -> None:
+def register_directive(name: str, directive: type[Directive]) -> None:
     """Register a directive to docutils.
 
     This modifies global state of docutils.  So it is better to use this
@@ -94,12 +107,12 @@ def unregister_role(name: str) -> None:
     roles._roles.pop(name, None)  # type: ignore
 
 
-def is_node_registered(node: Type[Element]) -> bool:
+def is_node_registered(node: type[Element]) -> bool:
     """Check the *node* is already registered."""
     return hasattr(nodes.GenericNodeVisitor, 'visit_' + node.__name__)
 
 
-def register_node(node: Type[Element]) -> None:
+def register_node(node: type[Element]) -> None:
     """Register a node to docutils.
 
     This modifies global state of some visitors.  So it is better to use this
@@ -110,7 +123,7 @@ def register_node(node: Type[Element]) -> None:
         additional_nodes.add(node)
 
 
-def unregister_node(node: Type[Element]) -> None:
+def unregister_node(node: type[Element]) -> None:
     """Unregister a node from docutils.
 
     This is inverse of ``nodes._add_nodes_class_names()``.
@@ -167,7 +180,7 @@ def patched_rst_get_language() -> Generator[None, None, None]:
 
 
 @contextmanager
-def using_user_docutils_conf(confdir: Optional[str]) -> Generator[None, None, None]:
+def using_user_docutils_conf(confdir: str | None) -> Generator[None, None, None]:
     """Let docutils know the location of ``docutils.conf`` for Sphinx."""
     try:
         docutilsconfig = os.environ.get('DOCUTILSCONFIG', None)
@@ -183,9 +196,45 @@ def using_user_docutils_conf(confdir: Optional[str]) -> Generator[None, None, No
 
 
 @contextmanager
-def patch_docutils(confdir: Optional[str] = None) -> Generator[None, None, None]:
+def du19_footnotes() -> Generator[None, None, None]:
+    def visit_footnote(self, node):
+        label_style = self.settings.footnote_references
+        if not isinstance(node.previous_sibling(), type(node)):
+            self.body.append(f'<aside class="footnote-list {label_style}">\n')
+        self.body.append(self.starttag(node, 'aside',
+                                       classes=[node.tagname, label_style],
+                                       role="note"))
+
+    def depart_footnote(self, node):
+        self.body.append('</aside>\n')
+        if not isinstance(node.next_node(descend=False, siblings=True),
+                          type(node)):
+            self.body.append('</aside>\n')
+
+    old_visit_footnote = HTMLTranslator.visit_footnote
+    old_depart_footnote = HTMLTranslator.depart_footnote
+
+    # Only apply on Docutils 0.18 or 0.18.1, as 0.17 and earlier used a <dl> based
+    # approach, and 0.19 and later use the fixed approach by default.
+    if docutils.__version_info__[:2] == (0, 18):
+        HTMLTranslator.visit_footnote = visit_footnote  # type: ignore[method-assign]
+        HTMLTranslator.depart_footnote = depart_footnote  # type: ignore[method-assign]
+
+    try:
+        yield
+    finally:
+        if docutils.__version_info__[:2] == (0, 18):
+            HTMLTranslator.visit_footnote = old_visit_footnote  # type: ignore[method-assign]
+            HTMLTranslator.depart_footnote = old_depart_footnote  # type: ignore[method-assign]
+
+
+@contextmanager
+def patch_docutils(confdir: str | None = None) -> Generator[None, None, None]:
     """Patch to docutils temporarily."""
-    with patched_get_language(), patched_rst_get_language(), using_user_docutils_conf(confdir):
+    with patched_get_language(), \
+         patched_rst_get_language(), \
+         using_user_docutils_conf(confdir), \
+         du19_footnotes():
         yield
 
 
@@ -203,7 +252,9 @@ class CustomReSTDispatcher:
     def __enter__(self) -> None:
         self.enable()
 
-    def __exit__(self, exc_type: Type[Exception], exc_value: Exception, traceback: Any) -> None:  # NOQA
+    def __exit__(
+        self, exc_type: type[Exception], exc_value: Exception, traceback: Any,
+    ) -> None:
         self.disable()
 
     def enable(self) -> None:
@@ -218,12 +269,13 @@ class CustomReSTDispatcher:
         roles.role = self.role_func
 
     def directive(self,
-                  directive_name: str, language_module: ModuleType, document: nodes.document
-                  ) -> Tuple[Optional[Type[Directive]], List[system_message]]:
+                  directive_name: str, language_module: ModuleType, document: nodes.document,
+                  ) -> tuple[type[Directive] | None, list[system_message]]:
         return self.directive_func(directive_name, language_module, document)
 
-    def role(self, role_name: str, language_module: ModuleType, lineno: int, reporter: Reporter
-             ) -> Tuple[RoleFunction, List[system_message]]:
+    def role(
+        self, role_name: str, language_module: ModuleType, lineno: int, reporter: Reporter,
+    ) -> tuple[RoleFunction, list[system_message]]:
         return self.role_func(role_name, language_module, lineno, reporter)
 
 
@@ -235,7 +287,7 @@ class sphinx_domains(CustomReSTDispatcher):
     """Monkey-patch directive and role dispatch, so that domain-specific
     markup takes precedence.
     """
-    def __init__(self, env: "BuildEnvironment") -> None:
+    def __init__(self, env: BuildEnvironment) -> None:
         self.env = env
         super().__init__()
 
@@ -270,15 +322,16 @@ class sphinx_domains(CustomReSTDispatcher):
         raise ElementLookupError
 
     def directive(self,
-                  directive_name: str, language_module: ModuleType, document: nodes.document
-                  ) -> Tuple[Optional[Type[Directive]], List[system_message]]:
+                  directive_name: str, language_module: ModuleType, document: nodes.document,
+                  ) -> tuple[type[Directive] | None, list[system_message]]:
         try:
             return self.lookup_domain_element('directive', directive_name)
         except ElementLookupError:
             return super().directive(directive_name, language_module, document)
 
-    def role(self, role_name: str, language_module: ModuleType, lineno: int, reporter: Reporter
-             ) -> Tuple[RoleFunction, List[system_message]]:
+    def role(
+        self, role_name: str, language_module: ModuleType, lineno: int, reporter: Reporter,
+    ) -> tuple[RoleFunction, list[system_message]]:
         try:
             return self.lookup_domain_element('role', role_name)
         except ElementLookupError:
@@ -298,7 +351,7 @@ class WarningStream:
 
 class LoggingReporter(Reporter):
     @classmethod
-    def from_reporter(cls, reporter: Reporter) -> "LoggingReporter":
+    def from_reporter(cls, reporter: Reporter) -> LoggingReporter:
         """Create an instance of LoggingReporter from other reporter object."""
         return cls(reporter.source, reporter.report_level, reporter.halt_level,
                    reporter.debug_flag, reporter.error_handler)
@@ -319,6 +372,8 @@ class NullReporter(Reporter):
 
 
 def is_html5_writer_available() -> bool:
+    from sphinx.deprecation import RemovedInSphinx70Warning
+
     warnings.warn('is_html5_writer_available() is deprecated.',
                   RemovedInSphinx70Warning)
     return True
@@ -334,7 +389,7 @@ def switch_source_input(state: State, content: StringList) -> Generator[None, No
         # replace it by new one
         state_machine = StateMachine([], None)
         state_machine.input_lines = content
-        state.memo.reporter.get_source_and_line = state_machine.get_source_and_line  # type: ignore  # NOQA
+        state.memo.reporter.get_source_and_line = state_machine.get_source_and_line  # type: ignore  # noqa: E501
 
         yield
     finally:
@@ -371,16 +426,16 @@ class SphinxDirective(Directive):
     """
 
     @property
-    def env(self) -> "BuildEnvironment":
+    def env(self) -> BuildEnvironment:
         """Reference to the :class:`.BuildEnvironment` object."""
         return self.state.document.settings.env
 
     @property
-    def config(self) -> "Config":
+    def config(self) -> Config:
         """Reference to the :class:`.Config` object."""
         return self.env.config
 
-    def get_source_info(self) -> Tuple[str, int]:
+    def get_source_info(self) -> tuple[str, int]:
         """Get source and line number."""
         return self.state_machine.get_source_and_line(self.lineno)
 
@@ -406,14 +461,14 @@ class SphinxRole:
     text: str           #: The interpreted text content.
     lineno: int         #: The line number where the interpreted text begins.
     inliner: Inliner    #: The ``docutils.parsers.rst.states.Inliner`` object.
-    options: Dict       #: A dictionary of directive options for customization
+    options: dict       #: A dictionary of directive options for customization
                         #: (from the "role" directive).
-    content: List[str]  #: A list of strings, the directive content for customization
+    content: list[str]  #: A list of strings, the directive content for customization
                         #: (from the "role" directive).
 
     def __call__(self, name: str, rawtext: str, text: str, lineno: int,
-                 inliner: Inliner, options: Dict = {}, content: List[str] = []
-                 ) -> Tuple[List[Node], List[system_message]]:
+                 inliner: Inliner, options: dict = {}, content: list[str] = [],
+                 ) -> tuple[list[Node], list[system_message]]:
         self.rawtext = rawtext
         self.text = unescape(text)
         self.lineno = lineno
@@ -433,20 +488,20 @@ class SphinxRole:
 
         return self.run()
 
-    def run(self) -> Tuple[List[Node], List[system_message]]:
+    def run(self) -> tuple[list[Node], list[system_message]]:
         raise NotImplementedError
 
     @property
-    def env(self) -> "BuildEnvironment":
+    def env(self) -> BuildEnvironment:
         """Reference to the :class:`.BuildEnvironment` object."""
         return self.inliner.document.settings.env
 
     @property
-    def config(self) -> "Config":
+    def config(self) -> Config:
         """Reference to the :class:`.Config` object."""
         return self.env.config
 
-    def get_source_info(self, lineno: int = None) -> Tuple[str, int]:
+    def get_source_info(self, lineno: int = None) -> tuple[str, int]:
         if lineno is None:
             lineno = self.lineno
         return self.inliner.reporter.get_source_and_line(lineno)  # type: ignore
@@ -475,8 +530,8 @@ class ReferenceRole(SphinxRole):
     explicit_title_re = re.compile(r'^(.+?)\s*(?<!\x00)<(.*?)>$', re.DOTALL)
 
     def __call__(self, name: str, rawtext: str, text: str, lineno: int,
-                 inliner: Inliner, options: Dict = {}, content: List[str] = []
-                 ) -> Tuple[List[Node], List[system_message]]:
+                 inliner: Inliner, options: dict = {}, content: list[str] = [],
+                 ) -> tuple[list[Node], list[system_message]]:
         # if the first character is a bang, don't cross-reference at all
         self.disabled = text.startswith('!')
 
@@ -505,7 +560,7 @@ class SphinxTranslator(nodes.NodeVisitor):
               This class is strongly coupled with Sphinx.
     """
 
-    def __init__(self, document: nodes.document, builder: "Builder") -> None:
+    def __init__(self, document: nodes.document, builder: Builder) -> None:
         super().__init__(document)
         self.builder = builder
         self.config = builder.config
@@ -549,19 +604,9 @@ class SphinxTranslator(nodes.NodeVisitor):
         logger.warning(__('unknown node type: %r'), node, location=node)
 
 
-# Node.findall() is a new interface to traverse a doctree since docutils-0.18.
-# This applies a patch docutils-0.17 or older to be available Node.findall()
-# method to use it from our codebase.
-if docutils.__version_info__ < (0, 18):
-    def findall(self, *args, **kwargs):
-        return iter(self.traverse(*args, **kwargs))
-
-    Node.findall = findall  # type: ignore
-
-
 # cache a vanilla instance of nodes.document
 # Used in new_document() function
-__document_cache__: Optional[nodes.document] = None
+__document_cache__: tuple[Values, Reporter]
 
 
 def new_document(source_path: str, settings: Any = None) -> nodes.document:
@@ -572,15 +617,18 @@ def new_document(source_path: str, settings: Any = None) -> nodes.document:
     This makes an instantiation of document nodes much faster.
     """
     global __document_cache__
-    if __document_cache__ is None:
-        __document_cache__ = docutils.utils.new_document(source_path)
+    try:
+        cached_settings, reporter = __document_cache__
+    except NameError:
+        doc = docutils.utils.new_document(source_path)
+        __document_cache__ = cached_settings, reporter = doc.settings, doc.reporter
 
     if settings is None:
-        # Make a copy of ``settings`` from cache to accelerate instantiation
-        settings = copy(__document_cache__.settings)
+        # Make a copy of the cached settings to accelerate instantiation
+        settings = copy(cached_settings)
 
     # Create a new instance of nodes.document using cached reporter
     from sphinx import addnodes
-    document = addnodes.document(settings, __document_cache__.reporter, source=source_path)
+    document = addnodes.document(settings, reporter, source=source_path)
     document.note_source(source_path, -1)
     return document
