@@ -4,11 +4,13 @@ Much of this code is adapted from Dave Kuhlman's "docpy" writer from his
 docutils sandbox.
 """
 
+from __future__ import annotations
+
 import re
 import warnings
 from collections import defaultdict
 from os import path
-from typing import TYPE_CHECKING, Any, Dict, Iterable, List, Optional, Set, Tuple, cast
+from typing import TYPE_CHECKING, Any, Iterable, cast
 
 from docutils import nodes, writers
 from docutils.nodes import Element, Node, Text
@@ -72,11 +74,11 @@ class LaTeXWriter(writers.Writer):
         ('Document class', ['--docclass'], {'default': 'manual'}),
         ('Author', ['--author'], {'default': ''}),
     ))
-    settings_defaults: Dict = {}
+    settings_defaults: dict[str, Any] = {}
 
     output = None
 
-    def __init__(self, builder: "LaTeXBuilder") -> None:
+    def __init__(self, builder: LaTeXBuilder) -> None:
         super().__init__()
         self.builder = builder
         self.theme: Theme = None
@@ -93,25 +95,41 @@ class Table:
     """A table data"""
 
     def __init__(self, node: Element) -> None:
-        self.header: List[str] = []
-        self.body: List[str] = []
+        self.header: list[str] = []
+        self.body: list[str] = []
         self.align = node.get('align', 'default')
-        self.classes: List[str] = node.get('classes', [])
+        self.classes: list[str] = node.get('classes', [])
+        self.styles: list[str] = []
+        if 'standard' in self.classes:
+            self.styles.append('standard')
+        elif 'borderless' in self.classes:
+            self.styles.append('borderless')
+        elif 'booktabs' in self.classes:
+            self.styles.append('booktabs')
+        if 'nocolorrows' in self.classes:
+            self.styles.append('nocolorrows')
+        elif 'colorrows' in self.classes:
+            self.styles.append('colorrows')
         self.colcount = 0
         self.colspec: str = None
-        self.colwidths: List[int] = []
+        self.colsep: str = None
+        if 'booktabs' in self.styles or 'borderless' in self.styles:
+            self.colsep = ''
+        elif 'standard' in self.styles:
+            self.colsep = '|'
+        self.colwidths: list[int] = []
         self.has_problematic = False
         self.has_oldproblematic = False
         self.has_verbatim = False
-        self.caption: List[str] = None
-        self.stubs: List[int] = []
+        self.caption: list[str] = None
+        self.stubs: list[int] = []
 
         # current position
         self.col = 0
         self.row = 0
 
         # A dict mapping a table location to a cell_id (cell = rectangular area)
-        self.cells: Dict[Tuple[int, int], int] = defaultdict(int)
+        self.cells: dict[tuple[int, int], int] = defaultdict(int)
         self.cell_id = 0  # last assigned cell_id
 
     def is_longtable(self) -> bool:
@@ -143,23 +161,30 @@ class Table:
 
         This is what LaTeX calls the 'preamble argument' of the used table environment.
 
-        .. note:: the ``\\X`` and ``T`` column type specifiers are defined in ``sphinx.sty``.
+        .. note::
+
+           The ``\\X`` and ``T`` column type specifiers are defined in
+           ``sphinxlatextables.sty``.
         """
         if self.colspec:
             return self.colspec
-        elif self.colwidths and 'colwidths-given' in self.classes:
+
+        _colsep = self.colsep
+        if self.colwidths and 'colwidths-given' in self.classes:
             total = sum(self.colwidths)
             colspecs = [r'\X{%d}{%d}' % (width, total) for width in self.colwidths]
-            return '{|%s|}' % '|'.join(colspecs) + CR
+            return f'{{{_colsep}{_colsep.join(colspecs)}{_colsep}}}' + CR
         elif self.has_problematic:
-            return r'{|*{%d}{\X{1}{%d}|}}' % (self.colcount, self.colcount) + CR
+            return r'{%s*{%d}{\X{1}{%d}%s}}' % (_colsep, self.colcount,
+                                                self.colcount, _colsep) + CR
         elif self.get_table_type() == 'tabulary':
             # sphinx.sty sets T to be J by default.
-            return '{|' + ('T|' * self.colcount) + '}' + CR
+            return '{' + _colsep + (('T' + _colsep) * self.colcount) + '}' + CR
         elif self.has_oldproblematic:
-            return r'{|*{%d}{\X{1}{%d}|}}' % (self.colcount, self.colcount) + CR
+            return r'{%s*{%d}{\X{1}{%d}%s}}' % (_colsep, self.colcount,
+                                                self.colcount, _colsep) + CR
         else:
-            return '{|' + ('l|' * self.colcount) + '}' + CR
+            return '{' + _colsep + (('l' + _colsep) * self.colcount) + '}' + CR
 
     def add_cell(self, height: int, width: int) -> None:
         """Adds a new cell to a table.
@@ -173,8 +198,8 @@ class Table:
                 self.cells[(self.row + row, self.col + col)] = self.cell_id
 
     def cell(
-        self, row: Optional[int] = None, col: Optional[int] = None
-    ) -> Optional["TableCell"]:
+        self, row: int | None = None, col: int | None = None,
+    ) -> TableCell | None:
         """Returns a cell object (i.e. rectangular area) containing given position.
 
         If no option arguments: ``row`` or ``col`` are given, the current position;
@@ -254,21 +279,21 @@ def rstdim_to_latexdim(width_str: str, scale: int = 100) -> str:
         elif unit == "%":
             res = r"%.5f\linewidth" % (amount_float / 100.0)
         else:
-            res = "%.5f%s" % (amount_float, unit)
+            res = f"{amount_float:.5f}{unit}"
     return res
 
 
 class LaTeXTranslator(SphinxTranslator):
-    builder: "LaTeXBuilder"
+    builder: LaTeXBuilder
 
     secnumdepth = 2  # legacy sphinxhowto.cls uses this, whereas article.cls
     # default is originally 3. For book/report, 2 is already LaTeX default.
     ignore_missing_images = False
 
-    def __init__(self, document: nodes.document, builder: "LaTeXBuilder",
-                 theme: "Theme") -> None:
+    def __init__(self, document: nodes.document, builder: LaTeXBuilder,
+                 theme: Theme) -> None:
         super().__init__(document, builder)
-        self.body: List[str] = []
+        self.body: list[str] = []
         self.theme = theme
 
         # flags
@@ -386,21 +411,21 @@ class LaTeXTranslator(SphinxTranslator):
 
         self.highlighter = highlighting.PygmentsBridge('latex', self.config.pygments_style,
                                                        latex_engine=self.config.latex_engine)
-        self.context: List[Any] = []
-        self.descstack: List[str] = []
-        self.tables: List[Table] = []
-        self.next_table_colspec: Optional[str] = None
-        self.bodystack: List[List[str]] = []
-        self.footnote_restricted: Optional[Element] = None
-        self.pending_footnotes: List[nodes.footnote_reference] = []
-        self.curfilestack: List[str] = []
-        self.handled_abbrs: Set[str] = set()
+        self.context: list[Any] = []
+        self.descstack: list[str] = []
+        self.tables: list[Table] = []
+        self.next_table_colspec: str | None = None
+        self.bodystack: list[list[str]] = []
+        self.footnote_restricted: Element | None = None
+        self.pending_footnotes: list[nodes.footnote_reference] = []
+        self.curfilestack: list[str] = []
+        self.handled_abbrs: set[str] = set()
 
-    def pushbody(self, newbody: List[str]) -> None:
+    def pushbody(self, newbody: list[str]) -> None:
         self.bodystack.append(self.body)
         self.body = newbody
 
-    def popbody(self) -> List[str]:
+    def popbody(self) -> list[str]:
         body = self.body
         self.body = self.bodystack.pop()
         return body
@@ -408,7 +433,7 @@ class LaTeXTranslator(SphinxTranslator):
     def astext(self) -> str:
         self.elements.update({
             'body': ''.join(self.body),
-            'indices': self.generate_indices()
+            'indices': self.generate_indices(),
         })
         return self.render('latex.tex_t', self.elements)
 
@@ -446,10 +471,10 @@ class LaTeXTranslator(SphinxTranslator):
             prefix = ''
             suffix = ''
 
-        return r'%s\renewcommand{%s}{%s}%s' % (prefix, command, definition, suffix) + CR
+        return fr'{prefix}\renewcommand{{{command}}}{{{definition}}}{suffix}' + CR
 
     def generate_indices(self) -> str:
-        def generate(content: List[Tuple[str, List[IndexEntry]]], collapsed: bool) -> None:
+        def generate(content: list[tuple[str, list[IndexEntry]]], collapsed: bool) -> None:
             ret.append(r'\begin{sphinxtheindex}' + CR)
             ret.append(r'\let\bigletter\sphinxstyleindexlettergroup' + CR)
             for i, (letter, entries) in enumerate(content):
@@ -474,7 +499,7 @@ class LaTeXTranslator(SphinxTranslator):
         if indices_config:
             for domain in self.builder.env.domains.values():
                 for indexcls in domain.indices:
-                    indexname = '%s-%s' % (domain.name, indexcls.name)
+                    indexname = f'{domain.name}-{indexcls.name}'
                     if isinstance(indices_config, list):
                         if indexname not in indices_config:
                             continue
@@ -487,7 +512,7 @@ class LaTeXTranslator(SphinxTranslator):
 
         return ''.join(ret)
 
-    def render(self, template_name: str, variables: Dict) -> str:
+    def render(self, template_name: str, variables: dict[str, Any]) -> str:
         renderer = LaTeXRenderer(latex_engine=self.config.latex_engine)
         for template_dir in self.config.templates_path:
             template = path.join(self.builder.confdir, template_dir,
@@ -498,7 +523,7 @@ class LaTeXTranslator(SphinxTranslator):
         return renderer.render(template_name, variables)
 
     @property
-    def table(self) -> Optional[Table]:
+    def table(self) -> Table | None:
         """Get current table."""
         if self.tables:
             return self.tables[-1]
@@ -591,7 +616,7 @@ class LaTeXTranslator(SphinxTranslator):
         if isinstance(parent, addnodes.seealso):
             # the environment already handles this
             raise nodes.SkipNode
-        elif isinstance(parent, nodes.section):
+        if isinstance(parent, nodes.section):
             if self.this_is_the_title:
                 if len(node.children) != 1 and not isinstance(node.children[0],
                                                               nodes.Text):
@@ -603,17 +628,16 @@ class LaTeXTranslator(SphinxTranslator):
                     self.elements['title'] = self.escape(node.astext())
                 self.this_is_the_title = 0
                 raise nodes.SkipNode
-            else:
-                short = ''
-                if any(node.findall(nodes.image)):
-                    short = ('[%s]' % self.escape(' '.join(clean_astext(node).split())))
+            short = ''
+            if any(node.findall(nodes.image)):
+                short = ('[%s]' % self.escape(' '.join(clean_astext(node).split())))
 
-                try:
-                    self.body.append(r'\%s%s{' % (self.sectionnames[self.sectionlevel], short))
-                except IndexError:
-                    # just use "subparagraph", it's not numbered anyway
-                    self.body.append(r'\%s%s{' % (self.sectionnames[-1], short))
-                self.context.append('}' + CR + self.hypertarget_to(node.parent))
+            try:
+                self.body.append(fr'\{self.sectionnames[self.sectionlevel]}{short}{{')
+            except IndexError:
+                # just use "subparagraph", it's not numbered anyway
+                self.body.append(fr'\{self.sectionnames[-1]}{short}{{')
+            self.context.append('}' + CR + self.hypertarget_to(node.parent))
         elif isinstance(parent, nodes.topic):
             self.body.append(r'\sphinxstyletopictitle{')
             self.context.append('}' + CR)
@@ -774,7 +798,7 @@ class LaTeXTranslator(SphinxTranslator):
         else:
             self.first_param = 0
         if not node.hasattr('noemph'):
-            self.body.append(r'\emph{')
+            self.body.append(r'\sphinxparam{')
 
     def depart_desc_parameter(self, node: Element) -> None:
         if not node.hasattr('noemph'):
@@ -796,11 +820,12 @@ class LaTeXTranslator(SphinxTranslator):
 
     def visit_seealso(self, node: Element) -> None:
         self.body.append(BLANKLINE)
-        self.body.append(r'\sphinxstrong{%s:}' % admonitionlabels['seealso'] + CR)
-        self.body.append(r'\nopagebreak' + BLANKLINE)
+        self.body.append(r'\begin{sphinxseealso}{%s:}' % admonitionlabels['seealso'] + CR)
 
     def depart_seealso(self, node: Element) -> None:
         self.body.append(BLANKLINE)
+        self.body.append(r'\end{sphinxseealso}')
+        self.body.append(BLANKLINE)
 
     def visit_rubric(self, node: Element) -> None:
         if len(node) == 1 and node.astext() in ('Footnotes', _('Footnotes')):
@@ -847,18 +872,28 @@ class LaTeXTranslator(SphinxTranslator):
                 raise UnsupportedError(
                     '%s:%s: longtable does not support nesting a table.' %
                     (self.curfilestack[-1], node.line or ''))
-            else:
-                # change type of parent table to tabular
-                # see https://groups.google.com/d/msg/sphinx-users/7m3NeOBixeo/9LKP2B4WBQAJ
-                self.table.has_problematic = True
+            # change type of parent table to tabular
+            # see https://groups.google.com/d/msg/sphinx-users/7m3NeOBixeo/9LKP2B4WBQAJ
+            self.table.has_problematic = True
         elif len(self.tables) > 2:
             raise UnsupportedError(
                 '%s:%s: deeply nested tables are not implemented.' %
                 (self.curfilestack[-1], node.line or ''))
 
         self.tables.append(Table(node))
+        if self.table.colsep is None:
+            self.table.colsep = '' if (
+                'booktabs' in self.builder.config.latex_table_style or
+                'borderless' in self.builder.config.latex_table_style
+            ) else '|'
         if self.next_table_colspec:
             self.table.colspec = '{%s}' % self.next_table_colspec + CR
+            if '|' in self.table.colspec:
+                self.table.styles.append('vlines')
+                self.table.colsep = '|'
+            else:
+                self.table.styles.append('novlines')
+                self.table.colsep = ''
             if 'colwidths-given' in node.get('classes', []):
                 logger.info(__('both tabularcolumns and :widths: option are given. '
                                ':widths: is ignored.'), location=node)
@@ -896,6 +931,8 @@ class LaTeXTranslator(SphinxTranslator):
         self.pushbody(self.table.header)
 
     def depart_thead(self, node: Element) -> None:
+        if self.body and self.body[-1] == r'\sphinxhline':
+            self.body.pop()
         self.popbody()
 
     def visit_tbody(self, node: Element) -> None:
@@ -903,42 +940,55 @@ class LaTeXTranslator(SphinxTranslator):
         self.pushbody(self.table.body)
 
     def depart_tbody(self, node: Element) -> None:
+        if self.body and self.body[-1] == r'\sphinxhline':
+            self.body.pop()
         self.popbody()
 
     def visit_row(self, node: Element) -> None:
         self.table.col = 0
-
+        _colsep = self.table.colsep
         # fill columns if the row starts with the bottom of multirow cell
         while True:
             cell = self.table.cell(self.table.row, self.table.col)
             if cell is None:  # not a bottom of multirow cell
                 break
-            else:  # a bottom of multirow cell
-                self.table.col += cell.width
-                if cell.col:
-                    self.body.append('&')
-                if cell.width == 1:
-                    # insert suitable strut for equalizing row heights in given multirow
-                    self.body.append(r'\sphinxtablestrut{%d}' % cell.cell_id)
-                else:  # use \multicolumn for wide multirow cell
-                    self.body.append(r'\multicolumn{%d}{|l|}{\sphinxtablestrut{%d}}' %
-                                     (cell.width, cell.cell_id))
+            # a bottom of multirow cell
+            self.table.col += cell.width
+            if cell.col:
+                self.body.append('&')
+            if cell.width == 1:
+                # insert suitable strut for equalizing row heights in given multirow
+                self.body.append(r'\sphinxtablestrut{%d}' % cell.cell_id)
+            else:  # use \multicolumn for wide multirow cell
+                self.body.append(r'\multicolumn{%d}{%sl%s}{\sphinxtablestrut{%d}}' %
+                                 (cell.width, _colsep, _colsep, cell.cell_id))
 
     def depart_row(self, node: Element) -> None:
         self.body.append(r'\\' + CR)
         cells = [self.table.cell(self.table.row, i) for i in range(self.table.colcount)]
         underlined = [cell.row + cell.height == self.table.row + 1 for cell in cells]
         if all(underlined):
-            self.body.append(r'\hline')
+            self.body.append(r'\sphinxhline')
         else:
             i = 0
             underlined.extend([False])  # sentinel
-            while i < len(underlined):
-                if underlined[i] is True:
-                    j = underlined[i:].index(False)
-                    self.body.append(r'\cline{%d-%d}' % (i + 1, i + j))
-                    i += j
+            if underlined[0] is False:
+                i = 1
+                while i < self.table.colcount and underlined[i] is False:
+                    if cells[i - 1].cell_id != cells[i].cell_id:
+                        self.body.append(r'\sphinxvlinecrossing{%d}' % i)
+                    i += 1
+            while i < self.table.colcount:
+                # each time here underlined[i] is True
+                j = underlined[i:].index(False)
+                self.body.append(r'\sphinxcline{%d-%d}' % (i + 1, i + j))
+                i += j
                 i += 1
+                while i < self.table.colcount and underlined[i] is False:
+                    if cells[i - 1].cell_id != cells[i].cell_id:
+                        self.body.append(r'\sphinxvlinecrossing{%d}' % i)
+                    i += 1
+            self.body.append(r'\sphinxfixclines{%d}' % self.table.colcount)
         self.table.row += 1
 
     def visit_entry(self, node: Element) -> None:
@@ -947,12 +997,14 @@ class LaTeXTranslator(SphinxTranslator):
         self.table.add_cell(node.get('morerows', 0) + 1, node.get('morecols', 0) + 1)
         cell = self.table.cell()
         context = ''
+        _colsep = self.table.colsep
         if cell.width > 1:
             if self.config.latex_use_latex_multicolumn:
                 if self.table.col == 0:
-                    self.body.append(r'\multicolumn{%d}{|l|}{%%' % cell.width + CR)
+                    self.body.append(r'\multicolumn{%d}{%sl%s}{%%' %
+                                     (cell.width, _colsep, _colsep) + CR)
                 else:
-                    self.body.append(r'\multicolumn{%d}{l|}{%%' % cell.width + CR)
+                    self.body.append(r'\multicolumn{%d}{l%s}{%%' % (cell.width, _colsep) + CR)
                 context = '}%' + CR
             else:
                 self.body.append(r'\sphinxstartmulticolumn{%d}%%' % cell.width + CR)
@@ -992,23 +1044,24 @@ class LaTeXTranslator(SphinxTranslator):
 
         cell = self.table.cell()
         self.table.col += cell.width
+        _colsep = self.table.colsep
 
         # fill columns if next ones are a bottom of wide-multirow cell
         while True:
             nextcell = self.table.cell()
             if nextcell is None:  # not a bottom of multirow cell
                 break
-            else:  # a bottom part of multirow cell
-                self.table.col += nextcell.width
-                self.body.append('&')
-                if nextcell.width == 1:
-                    # insert suitable strut for equalizing row heights in multirow
-                    # they also serve to clear colour panels which would hide the text
-                    self.body.append(r'\sphinxtablestrut{%d}' % nextcell.cell_id)
-                else:
-                    # use \multicolumn for wide multirow cell
-                    self.body.append(r'\multicolumn{%d}{l|}{\sphinxtablestrut{%d}}' %
-                                     (nextcell.width, nextcell.cell_id))
+            # a bottom part of multirow cell
+            self.body.append('&')
+            if nextcell.width == 1:
+                # insert suitable strut for equalizing row heights in multirow
+                # they also serve to clear colour panels which would hide the text
+                self.body.append(r'\sphinxtablestrut{%d}' % nextcell.cell_id)
+            else:
+                # use \multicolumn for not first row of wide multirow cell
+                self.body.append(r'\multicolumn{%d}{l%s}{\sphinxtablestrut{%d}}' %
+                                 (nextcell.width, _colsep, nextcell.cell_id))
+            self.table.col += nextcell.width
 
     def visit_acks(self, node: Element) -> None:
         # this is a list in the source, but should be rendered as a
@@ -1033,7 +1086,7 @@ class LaTeXTranslator(SphinxTranslator):
     def visit_enumerated_list(self, node: Element) -> None:
         def get_enumtype(node: Element) -> str:
             enumtype = node.get('enumtype', 'arabic')
-            if 'alpha' in enumtype and 26 < node.get('start', 0) + len(node):
+            if 'alpha' in enumtype and (node.get('start', 0) + len(node)) > 26:
                 # fallback to arabic if alphabet counter overflows
                 enumtype = 'arabic'
 
@@ -1187,7 +1240,7 @@ class LaTeXTranslator(SphinxTranslator):
         # self.body.append(r'\columnbreak\n')
         pass
 
-    def latex_image_length(self, width_str: str, scale: int = 100) -> Optional[str]:
+    def latex_image_length(self, width_str: str, scale: int = 100) -> str | None:
         try:
             return rstdim_to_latexdim(width_str, scale)
         except ValueError:
@@ -1199,8 +1252,8 @@ class LaTeXTranslator(SphinxTranslator):
         return isinstance(node.parent, nodes.TextElement)
 
     def visit_image(self, node: Element) -> None:
-        pre: List[str] = []  # in reverse order
-        post: List[str] = []
+        pre: list[str] = []  # in reverse order
+        post: list[str] = []
         include_graphics_options = []
         has_hyperlink = isinstance(node.parent, nodes.reference)
         if has_hyperlink:
@@ -1267,14 +1320,17 @@ class LaTeXTranslator(SphinxTranslator):
         if include_graphics_options:
             options = '[%s]' % ','.join(include_graphics_options)
         base, ext = path.splitext(uri)
+
         if self.in_title and base:
             # Lowercase tokens forcely because some fncychap themes capitalize
             # the options of \sphinxincludegraphics unexpectedly (ex. WIDTH=...).
-            self.body.append(r'\lowercase{\sphinxincludegraphics%s}{{%s}%s}' %
-                             (options, base, ext))
+            cmd = fr'\lowercase{{\sphinxincludegraphics{options}}}{{{{{base}}}{ext}}}'
         else:
-            self.body.append(r'\sphinxincludegraphics%s{{%s}%s}' %
-                             (options, base, ext))
+            cmd = fr'\sphinxincludegraphics{options}{{{{{base}}}{ext}}}'
+        # escape filepath for includegraphics, https://tex.stackexchange.com/a/202714/41112
+        if '#' in base:
+            cmd = r'{\catcode`\#=12' + cmd + '}'
+        self.body.append(cmd)
         self.body.extend(post)
 
     def depart_image(self, node: Element) -> None:
@@ -1308,7 +1364,10 @@ class LaTeXTranslator(SphinxTranslator):
             self.body.append(r'\begin{wrapfigure}{%s}{%s}' %
                              ('r' if node['align'] == 'right' else 'l', length or '0pt') + CR)
             self.body.append(r'\centering')
-            self.context.append(r'\end{wrapfigure}' + CR)
+            self.context.append(r'\end{wrapfigure}' +
+                                BLANKLINE +
+                                r'\mbox{}\par\vskip-\dimexpr\baselineskip+\parskip\relax' +
+                                CR)  # avoid disappearance if no text next issues/11079
         elif self.in_minipage:
             self.body.append(CR + r'\begin{center}')
             self.context.append(r'\end{center}' + CR)
@@ -1418,7 +1477,7 @@ class LaTeXTranslator(SphinxTranslator):
         domain = cast(StandardDomain, self.builder.env.get_domain('std'))
         if isinstance(next_node, HYPERLINK_SUPPORT_NODES):
             return
-        elif domain.get_enumerable_node_type(next_node) and domain.get_numfig_title(next_node):
+        if domain.get_enumerable_node_type(next_node) and domain.get_numfig_title(next_node):
             return
 
         if 'refuri' in node:
@@ -1432,7 +1491,26 @@ class LaTeXTranslator(SphinxTranslator):
                 pass
             else:
                 add_target(node['refid'])
-        for id in node['ids']:
+        # Temporary fix for https://github.com/sphinx-doc/sphinx/issues/11093
+        # TODO: investigate if a more elegant solution exists (see comments of #11093)
+        if node.get('ismod', False):
+            # Detect if the previous nodes are label targets. If so, remove
+            # the refid thereof from node['ids'] to avoid duplicated ids.
+            def has_dup_label(sib: Node | None) -> bool:
+                return isinstance(sib, nodes.target) and sib.get('refid') in node['ids']
+
+            prev = get_prev_node(node)
+            if has_dup_label(prev):
+                ids = node['ids'][:]  # copy to avoid side-effects
+                while has_dup_label(prev):
+                    ids.remove(prev['refid'])  # type: ignore
+                    prev = get_prev_node(prev)
+            else:
+                ids = iter(node['ids'])  # read-only iterator
+        else:
+            ids = iter(node['ids'])  # read-only iterator
+
+        for id in ids:
             add_target(id)
 
     def depart_target(self, node: Element) -> None:
@@ -1473,20 +1551,20 @@ class LaTeXTranslator(SphinxTranslator):
             try:
                 if type == 'single':
                     try:
-                        p1, p2 = [escape(x) for x in split_into(2, 'single', string)]
+                        p1, p2 = (escape(x) for x in split_into(2, 'single', string))
                         P1, P2 = style(p1), style(p2)
-                        self.body.append(r'\index{%s@%s!%s@%s%s}' % (p1, P1, p2, P2, m))
+                        self.body.append(fr'\index{{{p1}@{P1}!{p2}@{P2}{m}}}')
                     except ValueError:
                         p = escape(split_into(1, 'single', string)[0])
                         P = style(p)
-                        self.body.append(r'\index{%s@%s%s}' % (p, P, m))
+                        self.body.append(fr'\index{{{p}@{P}{m}}}')
                 elif type == 'pair':
-                    p1, p2 = [escape(x) for x in split_into(2, 'pair', string)]
+                    p1, p2 = (escape(x) for x in split_into(2, 'pair', string))
                     P1, P2 = style(p1), style(p2)
                     self.body.append(r'\index{%s@%s!%s@%s%s}\index{%s@%s!%s@%s%s}' %
                                      (p1, P1, p2, P2, m, p2, P2, p1, P1, m))
                 elif type == 'triple':
-                    p1, p2, p3 = [escape(x) for x in split_into(3, 'triple', string)]
+                    p1, p2, p3 = (escape(x) for x in split_into(3, 'triple', string))
                     P1, P2, P3 = style(p1), style(p2), style(p3)
                     self.body.append(
                         r'\index{%s@%s!%s %s@%s %s%s}'
@@ -1496,13 +1574,13 @@ class LaTeXTranslator(SphinxTranslator):
                          p2, P2, p3, p1, P3, P1, m,
                          p3, P3, p1, p2, P1, P2, m))
                 elif type == 'see':
-                    p1, p2 = [escape(x) for x in split_into(2, 'see', string)]
+                    p1, p2 = (escape(x) for x in split_into(2, 'see', string))
                     P1 = style(p1)
-                    self.body.append(r'\index{%s@%s|see{%s}}' % (p1, P1, p2))
+                    self.body.append(fr'\index{{{p1}@{P1}|see{{{p2}}}}}')
                 elif type == 'seealso':
-                    p1, p2 = [escape(x) for x in split_into(2, 'seealso', string)]
+                    p1, p2 = (escape(x) for x in split_into(2, 'seealso', string))
                     P1 = style(p1)
-                    self.body.append(r'\index{%s@%s|see{%s}}' % (p1, P1, p2))
+                    self.body.append(fr'\index{{{p1}@{P1}|see{{{p2}}}}}')
                 else:
                     logger.warning(__('unknown index entry type %s found'), type)
             except ValueError as err:
@@ -1536,7 +1614,7 @@ class LaTeXTranslator(SphinxTranslator):
             # references to labels in the same document
             id = self.curfilestack[-1] + ':' + uri[1:]
             self.body.append(self.hyperlink(id))
-            self.body.append(r'\emph{')
+            self.body.append(r'\sphinxsamedocref{')
             if self.config.latex_show_pagerefs and not \
                     self.in_production_list:
                 self.context.append('}}} (%s)' % self.hyperpageref(id))
@@ -1596,7 +1674,7 @@ class LaTeXTranslator(SphinxTranslator):
         else:
             # old style format (cf. "Fig.%{number}")
             text = escape_abbr(title) % (r'\ref{%s}' % self.idescape(id))
-        hyperref = r'\hyperref[%s]{%s}' % (self.idescape(id), text)
+        hyperref = fr'\hyperref[{self.idescape(id)}]{{{text}}}'
         self.body.append(hyperref)
 
         raise nodes.SkipNode
@@ -1678,8 +1756,8 @@ class LaTeXTranslator(SphinxTranslator):
 
     def visit_citation(self, node: Element) -> None:
         label = cast(nodes.label, node[0])
-        self.body.append(r'\bibitem[%s]{%s:%s}' % (self.encode(label.astext()),
-                                                   node['docname'], node['ids'][0]))
+        self.body.append(fr'\bibitem[{self.encode(label.astext())}]'
+                         fr'{{{node["docname"]}:{node["ids"][0]}}}')
 
     def depart_citation(self, node: Element) -> None:
         pass
@@ -1688,7 +1766,7 @@ class LaTeXTranslator(SphinxTranslator):
         if self.in_title:
             pass
         else:
-            self.body.append(r'\sphinxcite{%s:%s}' % (node['docname'], node['refname']))
+            self.body.append(fr'\sphinxcite{{{node["docname"]}:{node["refname"]}}}')
             raise nodes.SkipNode
 
     def depart_citation_reference(self, node: Element) -> None:
@@ -1710,8 +1788,8 @@ class LaTeXTranslator(SphinxTranslator):
         hlcode = self.highlighter.highlight_block(
             node.astext(), lang, opts=opts, location=node, nowrap=True)
         self.body.append(r'\sphinxcode{\sphinxupquote{%' + CR
-                         + hlcode.rstrip() + '%' + CR  # NoQA: W503
-                         + '}}')  # NoQA: W503
+                         + hlcode.rstrip() + '%' + CR
+                         + '}}')
         raise nodes.SkipNode
 
     def depart_literal(self, node: Element) -> None:
@@ -1763,7 +1841,7 @@ class LaTeXTranslator(SphinxTranslator):
 
             hlcode = self.highlighter.highlight_block(
                 node.rawsource, lang, opts=opts, linenos=linenos,
-                location=node, **highlight_args
+                location=node, **highlight_args,
             )
             if self.in_footnote:
                 self.body.append(CR + r'\sphinxSetupCodeBlockInFootnote')
@@ -2012,7 +2090,7 @@ class LaTeXTranslator(SphinxTranslator):
 
     def visit_math_block(self, node: Element) -> None:
         if node.get('label'):
-            label = "equation:%s:%s" % (node['docname'], node['label'])
+            label = f"equation:{node['docname']}:{node['label']}"
         else:
             label = None
 
@@ -2027,7 +2105,7 @@ class LaTeXTranslator(SphinxTranslator):
         raise nodes.SkipNode
 
     def visit_math_reference(self, node: Element) -> None:
-        label = "equation:%s:%s" % (node['docname'], node['target'])
+        label = f"equation:{node['docname']}:{node['target']}"
         eqref_format = self.config.math_eqref_format
         if eqref_format:
             try:
@@ -2044,7 +2122,7 @@ class LaTeXTranslator(SphinxTranslator):
         pass
 
     @property
-    def docclasses(self) -> Tuple[str, str]:
+    def docclasses(self) -> tuple[str, str]:
         """Prepends prefix to sphinx document classes"""
         warnings.warn('LaTeXWriter.docclasses() is deprecated.',
                       RemovedInSphinx70Warning, stacklevel=2)
@@ -2053,6 +2131,6 @@ class LaTeXTranslator(SphinxTranslator):
 
 # FIXME: Workaround to avoid circular import
 # refs: https://github.com/sphinx-doc/sphinx/issues/5433
-from sphinx.builders.latex.nodes import ( # NOQA isort:skip
+from sphinx.builders.latex.nodes import (  # noqa: E402  # isort:skip
     HYPERLINK_SUPPORT_NODES, captioned_literal_block, footnotetext,
 )
